package greenfoot.junitUtils;

import greenfoot.junitUtils.runner.TestRunnerState;

/**
 * It simulate random numbers generator for Test Driven Development (TDD) 
 * with JUnit. If we wants to verify the behaviour of any actor or 
 * world when it requests Greenfoot.getRandomNumber(maximum), the test
 * can act as system and generates a possible random number.
 *
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class Random {
    
	/**
	 * It queues a number in FIFO order for any next
	 * Greenfoot.getRandomNumber(maximun) operation. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRandom <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.getRandomNumber(10) == 7) {
	 *                 move(5);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the random number that has been generated produces the wanted behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.Random;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorRandomTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testRandomNumber() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorRandom actor = <b>new</b> ExampleActorRandom();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             Random.set(7);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(6, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param nextNumber will be the number returned by the operation
	 *                   Greenfoot.getRandomNumber(maximun).
	 *                   
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void set(int nextNumber) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        TestRunnerState.setRandom(nextNumber, Object.class);
	}

    /**
     * It queues a number in FIFO order for any next 
     * Greenfoot.getRandomNumber(maximun) operation 
     * from objects of specific class.<br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorRandom <b>extends</b> Actor {
	 *     
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.getRandomNumber(10) == 7) {
	 *                 move(5);
	 *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies that the random number that has been generated produces the wanted behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.Random;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorRandomTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testRandomNumber() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorRandom actor = <b>new</b> ExampleActorRandom();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             Random.set(7, ExampleActorRandom.<b>class</b>);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(6, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     *  
     * @param nextNumber will be the number returned by the operation
     *                   Greenfoot.getRandomNumber(maximun).
     *                   
     * @param callerClass is the class of object that can receive the number queued.
     *                    This class must be derived from Actor or World.
     *                    
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void set(int nextNumber, Class<?> callerClass) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        TestRunnerState.setRandom(nextNumber, callerClass);
    }
}
