/*
 This file is part of the BlueJ program. 
 Copyright (C) 1999-2009,2010,2012,2014,2016,2018,2019,2021  Michael Kolling and John Rosenberg
 
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version 2 
 of the License, or (at your option) any later version. 
 
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with this program; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 
 This file is subject to the Classpath exception as provided in the  
 LICENSE.txt file that accompanied this code.
 */
package greenfoot.junitUtils.runner;

import bluej.collect.DataCollector;
import bluej.debugger.DebuggerTestResult;
import bluej.debugger.jdi.TestResultsWithRunTime;
import bluej.pkgmgr.Project;
import bluej.pkgmgr.target.ClassTarget;
import bluej.pkgmgr.target.role.UnitTestClassRole;
import bluej.utility.Debug;
import greenfoot.guifx.GreenfootStage;
import javafx.application.Platform;
import threadchecker.OnThread;
import threadchecker.Tag;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

/**
 * Provide a thread class for running unit tests.
 * 
 * Unit tests are user code so they must be executed on a seperate thread.
 * This class provides the means to do this.
 * 
 * There are two primary modes of operation: run a single test (methodname != null),
 * and run all tests for a series of ClassTargets.
 * 
 * @author Davin McCall
 * 
 * @author francisco.guerra@ulpgc.es modifies it to include unit tests in Greenfoot
 */
public class TestRunnerThread extends Thread
{
    @OnThread(Tag.Worker)
    private final Iterator<ClassTarget> testIterator;
    private final GreenfootStage greenfootStage;

    private final String methodName; // Name of the test method; null to run all tests.
    
    private final Project project;

    /**
     * Construct a test runner thread for running multiple tests.
     */
    @OnThread(Tag.FXPlatform)
    public TestRunnerThread(GreenfootStage greenfootStage, Iterator<ClassTarget> i) {
        super("Test Runner");
        this.greenfootStage = greenfootStage;
        this.project = greenfootStage.getProject();
        this.methodName = null;
        testIterator = i;
   }
    
    /**
     * Construct a test runner thread for running a single test.
     */
    @OnThread(Tag.FXPlatform)
    public TestRunnerThread(GreenfootStage greenfootStage, ClassTarget ct, String methodName) {
        super("Test Runner");
        this.greenfootStage = greenfootStage;
        this.project = greenfootStage.getProject();
        List<ClassTarget> l = new ArrayList<ClassTarget>(1);
        l.add(ct);
        testIterator = l.iterator();
        this.methodName = methodName;
    }

    @Override
    @OnThread(value = Tag.Worker, ignoreParent = true)
    public void run(){
    	
        while (testIterator.hasNext()) {

            ClassTarget ct = testIterator.next();

            List<String> allMethods;
            if (methodName == null) {
                // Run all tests for a target, so find out what they are:
                CompletableFuture<List<String>> methodsFuture = new CompletableFuture<>();
                Platform.runLater(() -> startTestFindMethods(ct, methodsFuture));
                try
                {
                    allMethods = methodsFuture.get();
                }
                catch (InterruptedException | ExecutionException e)
                {
                    Debug.reportError(e);
                    allMethods = Collections.emptyList();
                }
                
            } else {
                // Run only a single test.
                allMethods = Arrays.asList(methodName);
            }

            // State 1 has given us the tests we need to run. Now run them:
            for (String methodName : allMethods){
                if (TestDisplayFrame.getTestDisplay().getDisplayClosed()) {
                    // User finalizes tests so have to tell the GreenfootStage that we're done:
                    Platform.runLater(() -> {
                            greenfootStage.endTestRun();
                    });
                	return;
                }
                TestResultsWithRunTime lastResult = 
                		project.getDebugger().runTestMethod(ct.getQualifiedName(), methodName);

                // Add the test result to the test display frame:
                Platform.runLater(() -> showNextResult(lastResult));
            }
        }

        // Finally, tell the GreenfootStage that we're done:
        Platform.runLater(() -> {
                greenfootStage.endTestRun();
        });
    }

    @OnThread(Tag.FXPlatform)
    private void showNextResult(TestResultsWithRunTime lastResult)
    {
    	for (DebuggerTestResult debuggerTestResult: lastResult.getResults()) {
    		TestDisplayFrame.getTestDisplay().addResult(debuggerTestResult);
    		DataCollector.testResult(greenfootStage.getPackage(), debuggerTestResult);
    	}
    }

    @OnThread(Tag.FXPlatform)
    private void startTestFindMethods(ClassTarget ct, CompletableFuture<List<String>> methodsFuture)
    {
        if (ct.isCompiled() && ct.isUnitTest() && ! ct.isAbstract() 
        		&& ct.getRole() instanceof UnitTestClassRole) 
        {
    		Class<?> cl = ct.getPackage().loadClass(ct.getQualifiedName());
    		List<String> allMethods = Arrays.stream(cl.getDeclaredMethods())
									    		    .filter(greenfootStage::isJUnitTestMethod)
									    		    .map(Method::getName)
									    		    .sorted()
									    		    .collect(Collectors.toList());
    		TestDisplayFrame.getTestDisplay().startTest(project, allMethods.size());
            methodsFuture.complete(allMethods);
            
        } else {
            methodsFuture.complete(Collections.emptyList());
        }
    }
    
}





